/* Shared routines

   OperStats - Copyright (c) 2000-2001 Darcy Grexton
   Contact: cygnus@uniserve.ca, skold @ HabberNet, ICQ #6285178

   This program is free but copyrighted software; see the file COPYING
   for details.
 */

#include "../inc/operstats.h"

/* Authenticate for SRA access. */
void do_auth (const char *whoami, const char *source)
{
    char *nick = strtok (NULL, " ");
    char *pass = strtok (NULL, " ");
    User *u;
    SRA *sra;
         
    if (!nick || !pass)
    {
	notice (whoami, source, RPL_SYNTAX, "AUTH NickName Password");
	notice (whoami, source, ERR_MORE_INFO, whoami, "AUTH");
	return;
    }

    u = finduser (source);
    sra = findsra (nick);

    if (!u)
	return;

    if (!sra)
    {
	notice (whoami, source, ERR_NOT_SRA);
	return;
    }

    /* Avoid excessive globals/logging. */
    if (is_sra (source))
    {
	notice (whoami, source, ERR_ALREADY_SRA);
	return;
    }

    if (stricmp (sra->nick, nick) || strcmp (sra->pass, pass))
    {
	notice (whoami, source, ERR_NOT_SRA);
	globops (whoami, SH_FAILED_AUTH, source, u->user, u->host);
	log ("Failed AUTH for %s from %s!%s@%s", sra->nick,
		source, u->user, u->host);
	return;
    }
    else
    {
	u->flags |= UF_SRA;
	notice (whoami, source, SH_NOW_SRA);
#ifdef UNREAL3
	send_cmd (whoami, "%s %s :is a Services Root Admin", me.token ? "BA" :
	    "SWHOIS", source);
#endif
	return;
    }
}

/* De-Authenticate from SRA access. */
void do_deauth (const char *whoami, const char *source)
{
    User *u;

    u = finduser (source);

    if (u->flags & UF_SRA)
    {
	u->flags &= ~UF_SRA;
	notice (whoami, source, SH_NO_LONGER_SRA);
	return;
    }

    /* We shouldn't get here, but just in case. */
    notice (whoami, source, ERR_NOT_SRA);
}

#ifdef RAWINJECT
/* Send raw info to the uplink */
void do_raw (const char *whoami, const char *source)
{
    char *text = strtok (NULL, "");

    if (!text)
    {
	notice (whoami, source, RPL_SYNTAX, "RAW Command");
	notice (whoami, source, ERR_MORE_INFO, whoami, "RAW");
    }
    else
	send_cmd (NULL, text);
}

/* Throw something into the inbuf and process it. */
void do_inject (const char *whoami, const char *source)
{
    char *text = strtok (NULL, "");

    if (!text)
    {
	notice (whoami, source, RPL_SYNTAX, "INJECT Data");
	notice (whoami, source, ERR_MORE_INFO, whoami, "INJECT");
    }
    else
    {
	strscpy (inbuf, text, BUFSIZE);
	process();
    }
}
#endif /* RAWINJECT */

/* Toggle debug mode. */
void do_debug (const char *whoami, const char *source)
{
    char *param = strtok (NULL, " ");
        
    if (!param)
    {
	notice (whoami, source, RPL_SYNTAX, "DEBUG Level");
	notice (whoami, source, ERR_MORE_INFO, whoami, "DEBUG");
	return;
    }

    if (atoi (param) > 0)
    {
	debuglevel = atoi (param);
	open_debug();
	debug ("-----------------------------------------------------");
	notice (whoami, source, SH_DEBUG_LEVEL, atoi (param),
	    debuglevel == 1 ? "Server<->Server traffic" :
	    debuglevel == 2 ? "Some function calls" :
	    debuglevel == 3 ? "Server<->Server traffic and some function calls"
	    : "Unknown");
    }
    else
    {
	notice (whoami, source, SH_DEBUG_LEVEL, atoi (param), "Off");
	debug ("%s turned debug mode off.", source);
	debuglevel = 0;
	close_debug();
    }
}

/* Terminate operstats permanently. */
void do_shutdown (const char *whoami, const char *source)
{
    char reason[BUFSIZE], orig[BUFSIZE], *param;

    param = strtok (NULL, "");

    if (!param)
	memcpy (reason, "No reason given", sizeof (reason));
    else
	memcpy (reason, param, sizeof (reason));

    strscpy (orig, reason, sizeof (orig));

    param = strtok (reason, " ");

    /* Should we skip the DB save? */
    if (!stricmp (param, "NOSAVE"))
    {
	/* Yep. */
	param = strtok (NULL, "");
	snprintf (quitmsg, BUFSIZE, param);

	if (!quitmsg)
	    snprintf (quitmsg, BUFSIZE, "No reason given");

	globops (whoami, SH_SHUTTING_DOWN, source, quitmsg);
	runflags |= (RUN_QUITTING | RUN_TERMINATING | RUN_SHUTDOWN);
    }
    else
    {
	/* Nope. Save the DBs. */
	snprintf (quitmsg, BUFSIZE, orig);

	if (statserv_on == TRUE)
	    save_ss_dbase ();

	/* Check for any expired AKills */
	if (operserv_on == TRUE)
	{
	    expire_akills ();
	    save_os_dbase ();
	}

	globops (whoami, SH_SHUTTING_DOWN, source, quitmsg);
	runflags |= (RUN_QUITTING | RUN_TERMINATING | RUN_SHUTDOWN);
    }
}

/* Restart OperStats. */
void do_restart (const char *whoami, const char *source)
{
    char reason[BUFSIZE], orig[BUFSIZE], *param;

    param = strtok (NULL, "");

    if (!param)
	memcpy (reason, "No reason given", sizeof (reason));
    else
	memcpy (reason, param, sizeof (reason));

    strscpy (orig, reason, sizeof (orig));

    param = strtok (reason, " ");

    /* If wait_restart isn't set, OperStats won't restart. But if we're
       using this function, we want them to restart, so we set
       wait_restart to 5 seconds here. This change is wiped out when
       they restart.
     */
    wait_restart = 5;

    /* Should we skip the DB save? */
    if (!stricmp (param, "NOSAVE"))
    {
	/* Yep. */
	param = strtok (NULL, "");
	snprintf (quitmsg, BUFSIZE, param);

	if (!quitmsg)
	    snprintf (quitmsg, BUFSIZE, "No reason given");

	globops (whoami, SH_RESTARTING, duration (wait_restart), source,
	    quitmsg);
	runflags |= (RUN_QUITTING | RUN_TERMINATING);
    }
    else
    {
	/* Nope. Save the DBs. */
	snprintf (quitmsg, BUFSIZE, orig);

	if (statserv_on == TRUE)
	    save_ss_dbase ();

	/* Check for any expired AKills */
	if (operserv_on == TRUE)
	{
	    expire_akills ();
	    save_os_dbase ();
	}

	globops (whoami, SH_RESTARTING, duration (wait_restart), source,
	    quitmsg);
	runflags |= (RUN_QUITTING | RUN_TERMINATING);
    }
}

/* Show various OperStats settings */
void do_settings (const char *whoami, const char *source)
{
    notice (whoami, source, SH_SETTINGS_START);
    notice (whoami, source, " ");
#ifdef UNREAL3
    notice (whoami, source, SH_SETTINGS_UNREAL, unreal_numeric);
#endif
    notice (whoami, source, SH_SETTINGS_SYNC, duration (update_timeout));
    if (strlen (snoopchan))
	notice (whoami, source, SH_SETTINGS_SNOOPCHAN, snoopchan);
    if (strlen (joinchan))
	notice (whoami, source, SH_SETTINGS_JOINCHAN, joinchan);
    notice (whoami, source, SH_SETTINGS_RESTART, duration (wait_restart));
    notice (whoami, source, SH_SETTINGS_DEBUG, debuglevel,
	debuglevel ? "" : " (Off)");
    notice (whoami, source, SH_SETTINGS_LOGUPDATES,
	logupdates_on == TRUE ? "On" : "Off");
    if (operserv_on == TRUE)
    {
	notice (whoami, source, SH_SETTINGS_AKILL, duration (def_akill_time));
	notice (whoami, source, SH_SETTINGS_AKILLREASON, def_akill_reason);
	notice (whoami, source, SH_SETTINGS_AKILLGLOBAL,
	    globalonakill_on == TRUE ? "On" : "Off");
	notice (whoami, source, SH_SETTINGS_AKILLLEARN,
	    learnakills_on == TRUE ? "On" : "Off");
	notice (whoami, source, SH_SETTINGS_AKILLFORGET,
	    forgetakills_on == TRUE ? "On" : "Off");
	notice (whoami, source, SH_SETTINGS_NONWILD, nonwildreq);
	notice (whoami, source, SH_SETTINGS_DETECTION, maxclones ? "On" :
	    "Off");
	if (maxclones)
	{
	    notice (whoami, source, SH_SETTINGS_CLONES, maxclones);
	    notice (whoami, source, SH_SETTINGS_CLONEKILLS, maxclonekills);
	}
	if (globalnoticer_on == TRUE)
	{
	    notice (whoami, source, SH_SETTINGS_TLDS, tlds);
	    notice (whoami, source, SH_SETTINGS_STAMPLOGON,
		stamplogons_on == TRUE ? "On" : "Off");
	}
    }
    if (statserv_on == TRUE)
    {
	notice (whoami, source, SH_SETTINGS_PING, duration (ping_delay));
	notice (whoami, source, SH_SETTINGS_DUMPCHANS,
	    dumpchans_on == TRUE ? "On" : "Off");
	notice (whoami, source, SH_SETTINGS_JAVACLIENT,
	    javaclient_on == TRUE ? "On" : "Off");
	notice (whoami, source, SH_SETTINGS_DUMPSTATS,
	    dumpstats_on == TRUE ? "On" : "Off");
	notice (whoami, source, SH_SETTINGS_LAGGLOBAL,
	    globalonlag_on == TRUE ? "On" : "Off");
	notice (whoami, source, SH_SETTINGS_LAGTHRESH,
	    duration (lagthreshold));
    }
    notice (whoami, source, " ");
    notice (whoami, source, SH_SETTINGS_END);
}

/* Update the DBs. */
void do_update (const char *whoami, const char *source)
{
    globops (whoami, SH_FORCE_UPDATE, source);
    runflags |= RUN_SAVEDBS;
}

/* Re-read operstats.conf */
void do_rehash (const char *whoami, const char *source)
{
    /* Just in case. */
    do_update (source, whoami);

    globops (whoami, SH_DOING_REHASH, CONFIGFILE, source);

    /* Turn these off. They'll be re-enabled if they're
       turned on in the conf.
     */
    operserv_on = FALSE;
    statserv_on = FALSE;
    globalnoticer_on = FALSE;
    globalonakill_on = FALSE;
    stamplogons_on = FALSE;
    logupdates_on = FALSE;
    dumpchans_on = FALSE;
    javaclient_on = FALSE;
    dumpstats_on = FALSE;
    learnakills_on = FALSE;
    dumptlds_on = FALSE;
    forgetakills_on = FALSE;
    globalonlag_on = FALSE;
    maxclones = 0;
    maxclonekills = 0;

    /* Flush TLDs, SRAs, and TimeZones (some might be removed) */
    flushlists ();

    /* Now re-read. */
    load_conf ();
    
    /* Now check it */
    check_conf(); 

    /* Remove any disabled pseudoclients */
    if (ss.isonline && statserv_on == FALSE)
    {
	send_cmd (s_StatServ, "%s :Client disabled", me.token ? "," : "QUIT");
	--userstat;
	ss.isonline = 0;
    }

    if (os.isonline && operserv_on == FALSE)
    {
	send_cmd (whoami, "%s :Client disabled", me.token ? "," : "QUIT");
	--userstat;
	os.isonline = 0;
    }

    if (gn.isonline && globalnoticer_on == FALSE)
    {
	send_cmd (s_GlobalNoticer, "%s :Client disabled", me.token ? "," :
	    "QUIT");
	--userstat;
	gn.isonline = 0;
    }

    /* Re-read StatServ stuff too */
    if (statserv_on == TRUE)
    {
	load_tlds ();
	load_zones ();
    }
}
